#!/usr/bin/ruby
#
# Author: Mateusz Pawlowski <js+muninplugins@yllq.net>
#
# Distributed under: GNU GPL 3 available at http://www.gnu.org/licenses/gpl.txt
#
# This script is designed to work with:
# 	Munin Version: 1.3.4 ( may work with 1.3.3 )
# 	Cherokee Version: 0.98.1
#
#%# family=contrib
# Cherokee has to have $host/about enabled with server info set to normal
# Or you can change the variables below as you wish

$about_location = "/about/ruby/info"

# Handling arguments
#
# cheroke_host_type_data Shows data transferred
# cheroke_host_type_rate Shows data rate transfer
# cheroke_host_type_mod Shows Modules
# cheroke_host_type_conn Shows connections

 require 'net/http'


#assigning global variables from ENV
$host = ENV["host"]
$type = "data"


#assigning global variables from the script file name
if $0 =~ /^(?:|.*\/)cherokee_([^_]+)_type_(.+)$/
    $host = $1
    $type = $2
    if $host =~ /^([^:]+):(\d+)$/

	$host = $1
	$type = $2
    end
end

#this method will parse the body and get values out
def output_values(response)
  case $type
  when "mod"
    section = response["modules"]
    section.each { |key,value|
      puts "#{key}.value #{value}"
    }

  when "data"
    section = response["traffic"]
    puts "sent.value #{section["tx"]}"
    puts "received.value #{section["rx"]}"

  when "rate"
    section = response["traffic"]
    puts "sent.value #{section["tx"]}"
    puts "received.value #{section["rx"]}"

  when "conn"
    section = response["connections"]
    section.each { |key,value|
      puts "#{key}.value #{value}"
    }
  end
end


def get_data
    #requesting data from the Cherokee about webpage
  url = URI.parse("http://#{$host}#{$about_location}")
  req = Net::HTTP::Get.new(url.path)
  res = Net::HTTP.start(url.host, url.port) {|http|
    http.request(req)
  }
  object = eval res.body.to_s

  return object
end

def munin_values(res)

  #evaluate returned string as code
  output_values(res)

end

#responding to "config" argument
def munin_config(response)
  #puts "host_name #{$host}"
  puts "graph_category cherokee"
  case $type
  when "rate"
    puts "graph_title Cherokee Data Transfer Rate"
    puts "graph_vlabel Bits sent(+) / received(-) per ${graph_period}"
    puts "graph_args --base 1000"
    puts "graph_info"
    puts "received.label Received"
    puts "received.graph no"
    puts "received.type COUNTER"
    puts "received.cdef received,8,*"
    puts "sent.label bps"
    puts "sent.type COUNTER"
    puts "sent.negative received"
    puts "sent.cdef sent,8,*"
  when "conn"
    puts "graph_title Cherokee Connections"
    puts "graph_vlabel Connections"
    puts "graph_args --base 1000 --logarithmic"
    response["connections"].each_key { |key|
      puts "#{key}.label #{key.capitalize}"
      puts "#{key}.draw AREASTACK"
    }
  when "data"
    puts "graph_title Cherokee Data Transferred"
    puts "graph_vlabel Bytes"
    puts "sent.label Sent"
    puts "received.label Received"
  when "mod"
    puts "graph_title Cherokee Loaded Modules"
    puts "graph_vlabel Modules"
    puts "graph_args --base 1000 --lower-limit 0"
    response["modules"].each_key { |key|
      puts "#{key}.label #{key.capitalize}"
      puts "#{key}.draw AREASTACK"
    }

  end
end

case ARGV[0]
when "config"
  munin_config(get_data)
else
  munin_values(get_data)
end
